import { ElNotification as message } from 'element-plus';
import { Session } from '/@/utils/storage';

const websocket: any = {
	websocket: null,
	connectURL: import.meta.env.VITE_VOICE_ASSISTANT_SOCKET_URL,
	// 开启标识
	socket_open: false,
	// 心跳timer
	hearBeat_timer: null,
	// 心跳发送频率
	hearBeat_interval: 2 * 1000,
	// 是否自动重连
	is_reconnect: true,
	// 重连次数
	reconnect_count: 3,
	// 已发起重连次数
	reconnect_current: 1,
	// 重连timer
	reconnect_timer: null,
	// 重连频率
	reconnect_interval: 5 * 1000,
	init: (receiveMessage: Function | null) => {
		if (!('WebSocket' in window)) {
			message.warning('浏览器不支持WebSocket');
			return null;
		}
		const token = Session.get('token');
		if (!token) {
			message.warning('websocket认证失败');
			return null;
		}
		const wsUrl = `${import.meta.env.VITE_VOICE_ASSISTANT_SOCKET_URL}${token}/`;
		websocket.websocket = new WebSocket(wsUrl);
		websocket.websocket.onmessage = (e: any) => {
			if (receiveMessage) {
				receiveMessage(e);
			}
		};
		websocket.websocket.onclose = (e: any) => {
			websocket.socket_open = false;
			// 需要重新连接
			if (websocket.is_reconnect) {
				websocket.reconnect_timer = setTimeout(() => {
					// 超过重连次数
					if (websocket.reconnect_current > websocket.reconnect_count) {
						clearTimeout(websocket.reconnect_timer);
						websocket.is_reconnect = false;
						return;
					}
					// 记录重连次数
					websocket.reconnect_current++;
					websocket.reconnect();
				}, websocket.reconnect_interval);
			}
		};
		// 连接成功
		websocket.websocket.onopen = function () {
			websocket.socket_open = true;
			websocket.is_reconnect = true;
			// 开启心跳
			websocket.heartbeat();
		};
		// 连接发生错误
		websocket.websocket.onerror = function () {};
	},
	heartbeat: () => {
		websocket.hearBeat_timer && clearInterval(websocket.hearBeat_timer);

		websocket.hearBeat_timer = setInterval(() => {
			let data = {
				token: Session.get('token'),
			};
			websocket.send(data);
		}, websocket.hearBeat_interval);
	},
	send: (data: { token: any }, callback = null) => {
		// 开启状态直接发送
		if (websocket.websocket.readyState === websocket.websocket.OPEN) {
			websocket.websocket.send(JSON.stringify(data));
			// @ts-ignore
			callback && callback();
		} else {
			clearInterval(websocket.hearBeat_timer);
			message({
				type: 'warning',
				message: 'socket链接已断开',
				duration: 1000,
			});
		}
	},
	close: () => {
		websocket.is_reconnect = false;
		websocket.websocket.close();
		websocket.websocket = null;
	},
	/**
	 * 重新连接
	 */
	reconnect: () => {
		if (websocket.websocket && !websocket.is_reconnect) {
			websocket.close();
		}
		websocket.init(null);
	},
};
export default websocket;
